﻿#include "Matrix.h"

Matrix operator+(const Matrix& matrix_1, const Matrix& matrix_2)
{
	Matrix output(matrix_1.m_rows, matrix_1.m_cols);
	for (uint32_t i = 0; i < output.m_rows; i++) {
		for (uint32_t j = 0; j < output.m_cols; j++)
			output.at(i, j) = matrix_1.at(i, j) + matrix_2.at(i,j);
	}
	return output;
}

Matrix operator+(const Matrix& matrix_1, const double in){
	Matrix buff{ matrix_1.m_rows, matrix_1.m_cols, in };
	return (matrix_1+buff);
}

Matrix operator+(const double in, const Matrix& matrix_1){
	return (matrix_1 + in);
}

Matrix operator-(const Matrix& matrix_1, const Matrix& matrix_2)
{
	Matrix output(matrix_1.m_rows, matrix_1.m_cols);
	for (uint32_t i = 0; i < output.m_rows; i++) {
		for (uint32_t j = 0; j < output.m_cols; j++)
			output.at(i, j) = matrix_1.at(i, j) - matrix_2.at(i, j);
	}
	return output;
}

Matrix operator-(const Matrix& matrix_1, const double in) {
	Matrix buff{ matrix_1.m_rows, matrix_1.m_cols, in };
	return (matrix_1 - buff);
}

Matrix operator-(const double in, const Matrix& matrix_1) {
	
	Matrix buff{ matrix_1.m_rows, matrix_1.m_cols, in };
	return (buff - matrix_1);
}

Matrix operator*(const Matrix& matrix_1, const Matrix& matrix_2) {
	assert((matrix_1.m_cols == matrix_2.m_rows));
	Matrix output{ matrix_1.m_rows, matrix_2.m_cols };
	for (uint32_t i = 0; i < output.m_rows; i++) {
		for (uint32_t j = 0; j < output.m_cols; j++) {
			double result = 0; 
			for (uint32_t k = 0; k < matrix_1.m_cols; k++) 
				result += matrix_1.at(i, k) * matrix_2.at(k, j);
			output.at(i, j) = result;
		}
	}
	return output;
}

Matrix operator*(const Matrix& matrix_1, const double in) {
	Matrix output{ matrix_1.m_rows, matrix_1.m_cols };
	for (uint32_t i = 0; i < output.m_rows; i++) {
		for (uint32_t j = 0; j < output.m_cols; j++) {
			output.at(i, j) = matrix_1.at(i, j) * in;
		}
	}
	return output;
}

Matrix operator*(const double in, const Matrix& matrix_1) {
		return (matrix_1*in);
}

Matrix operator/(const Matrix& matrix_1, const double in) {
	return (matrix_1 * (1 / in));
}

Matrix operator/(const double in, const Matrix& matrix_1) {
	return ((1 / in) * matrix_1);
}

Matrix Matrix::operator=(const Matrix& input_matrix)
{
	if (this == &input_matrix) return *this;

	m_cols = input_matrix.m_cols;
	m_rows = input_matrix.m_rows;
	m_matrix = input_matrix.m_matrix;
	return *this;
}


std::ostream& operator<<(std::ostream& out, const Matrix& matrix_1) {
	out << "\n";
	for (uint32_t i = 0; i < matrix_1.m_rows; i++) {
		for (uint32_t j = 0; j < matrix_1.m_cols; j++) {
			out << matrix_1(i, j) << " ";
		}
		out << "\n";
	}
	return out;
}

//std::ofstream& operator<<(std::ofstream& out, const Matrix& i_matrix)
//{
//	out << "\n";
//	for (uint32_t i = 0; i < i_matrix.m_rows; i++) {
//		for (uint32_t j = 0; j <i_matrix.m_cols; j++) {
//			out << i_matrix.at(i, j) << " ";
//		}
//		out << "\n";
//	}
//	return out;
//}

double& Matrix::operator()(uint32_t input_rows, uint32_t input_cols)
{
	return (at(input_rows, input_cols));
}

double Matrix::operator()(uint32_t input_rows, uint32_t input_cols) const
{
	return (at(input_rows, input_cols));
}


Matrix Matrix::Transpose()
{
	Matrix output{m_cols, m_rows };
	for (uint32_t i = 0; i < output.m_rows; i++) {
		for (uint32_t j = 0; j < output.m_cols; j++)
			output.at(i, j) = at(j, i);
	}
	return output;
}

Matrix Matrix::RotationOX(double α)
{
	return Matrix(3, 3, {1,0,0,0,cos(α), sin(α),0,-sin(α), cos(α)});
}
Matrix Matrix::RotationOY(double α)
{
	return Matrix(3, 3, { cos(α),0,-sin(α),0,1,0,sin(α),0, cos(α) });
}
Matrix Matrix::RotationOZ(double α)
{
	return Matrix(3, 3, {cos(α), sin(α), 0,-sin(α), cos(α), 0,0,0,1});

}